/****************************************/
/* Cool Herders                         */
/* A demented muse by Brendan Wiese     */
/* http://starbow.org                   */
/* Programmed by Mike Brent             */
/* http://tursi.yiffco.com              */
/* cool.c                               */
/****************************************/

#pragma align 4

#include <stdio.h>
#include <malloc.h>
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/fs.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/spu.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/sndsrv_abi.h>
#include <kallisti/irq.h>
#include <kallisti/vmu.h>
#include <kallisti/biosfont.h>
#include "../kosh/abi/conio.h"

#include "sound.h"
#include "sprite.h"
#include "cool.h"
#include "tursivmu.h"
#include "bmp.h"
#include "rand.h"
#include "levels.h"
#include "collide.h"
#include "sheep.h"
#include "font.h"

#define VOLUME 240
#define PAN 128

#define SHEEPVOL 128
#define CAUGHT 0
#define ZAPPED 5
#define FREED 3
#define START 1
#define ANY1  2
#define ANY2  4

#define SNDDELAY 15

void do_screen();
void load_data();

/* KallistOS interfaces */
abi_thread_t	*thread		= NULL;
abi_maple_t		*maple 		= NULL;
abi_ta_t		*ta	 		= NULL;
abi_video_t		*video		= NULL;
abi_fs_t		*fs			= NULL;
abi_spu_t		*spu		= NULL;
abi_conio_t		*conio		= NULL;
abi_sndsrv_t	*sound		= NULL;

/* game variables */
uint32 nFrames, nFirstJiffy, nLastJiffy, nGameFrames;
uint8 addr ;
uint32 txr_title, txr_sprites, txr_winner[4];
uint32 txr_levela, txr_levelb, txr_levelc;
uint32 txr_sheep, txr_smfont, txr_lgfont;
uint32 txr_readygopause, txr_herder[4];
uint32 txr_controlwork;
int    fMusic, fSound;
char   OldVMU[256] ;
char   levelbuf[LEVELBYTES];
int    idx, inDemo;
int	   sndCountdown;
BITMAP *myBmp;
int    ControllerState[4];		// Array that contains whether each port has a valid controller

extern int snd_sheep[6];
extern int siz_sheep[6];
extern int title_song;
extern int mid_song;
extern int win_song;
extern int no_song;

HerdStruct herder[4] ;

#define HERDERMAXSPEED 3
#define MAXLIGHTNING 5

int HERDERSPEED  = 1;		// This may be changable later
int SHEEPSPEED   = 3;

int PLAYFIELDXOFF  = -16;	// Centering the playfield
int PLAYFIELDYOFF  = -16;	

/*****************/

int main(int argc, char *argv[]) {
	int idx;
	char buf[256];
	int x, y, c;
	uint16 val;
	cont_cond_t cond;
	uint32 unit;

	printf("Initializing Cool Herders v0.3...\n");

	memset(herder, 0, sizeof(herder));

	/* init all our abi's */
	thread = (abi_thread_t*)lib_open("thread");
	if (!thread) { printf("Can't open thread library\r\n"); return -1; }
	spu = (abi_spu_t*)lib_open("spu");
	if (spu == NULL) { printf("Can't open SPU lib\r\n"); return -1; }
	maple = (abi_maple_t*)lib_open("maple");
	if (!maple) { printf("Can't open maple library\r\n"); return -1; }
	ta = (abi_ta_t*)lib_open("ta");
	if (!ta) { printf("Can't open ta library\r\n"); return -1; }
	video = (abi_video_t*)lib_open("video");
	if (!video) { printf("Can't open video library\r\n"); return -1; }
	fs = (abi_fs_t*)lib_open("fs");
	if (!fs) { printf("Can't open fs library\r\n"); return -1; }
	sound = (abi_sndsrv_t*)lib_open("sndsrv");
	if (!sound) { printf("Can't open sound server library\r\n"); return -1; }
	conio = lib_open("conio");
	if (conio != NULL) {
		printf("Kosh detected; freezing console\r\n");
		conio->freeze();
	}

	// Display initial copyright screen
	do_screen();

	ReScanVMUs();

	inDemo=0;
	fMusic=1;
	fSound=1;
	sndCountdown=0;

	if (maple->first_controller()!=0) {
		if (fMusic) {
			sound_start(title_song, false);
		}
	}

	// Make sure we're up to date
	ReScanVMUs();
	printf("Waiting for START...\n");
	nFrames=0;
	while(1) {
		// now to get it onscreen
		ta->begin_render();

		// Opaque polygons
		AddPicture(txr_title, 511.0f);
		ta->commit_eol();

		// Translucent polygons
		if (maple->first_controller()==0) {
			// Attach Controller
			addPage(txr_controlwork, 0, 0, 192, 48);
			ta->commit_eol();
			ta->finish_frame();
			thread->sleep(500);
			maple->rescan_bus(0);
			// Make sure we're up to date
			ReScanVMUs();
			continue;
		} else {
			registerDummy(TA_TRANSLUCENT);
		}

		// Work In Progress
		addPage2(txr_controlwork, 365, 370, 0, 96, 240, 191, nFrames%200-100);

		ta->commit_eol();
		ta->finish_frame();

		cond.buttons=~(0);
		nFrames++;
		if ((nFrames%180)==0) {
			ReScanVMUs();
		} else {
			for (idx=0; idx<4; idx++) {
				if (herder[idx].address!=0) {
					if (maple->cont_get_cond(herder[idx].address, &cond) < 0) {
						printf("Controller read error.\n");
						thread->sleep(500);
						maple->rescan_bus(0);
						// Make sure we're up to date
						ReScanVMUs();
						continue;
					}
					if (~cond.buttons) break;
				}
			}
		}

		if (!(cond.buttons & CONT_START)) {
			// Seed random number generator
			srand(thread->jiffies());

			printf("Game Begins... A+B+X+Y to quit\n");

			if (Game()) {
				break;
			}

			nFrames=0;
			if (fMusic) {
				sound_start(title_song, false);
			}
			continue;
		}

		if ((cond.buttons & (CONT_A | CONT_B | CONT_X | CONT_Y)) == 0) {
			printf("Exit outer...\n");
			break;
		}

		if (nFrames%1800==0) {
			for (idx=0; idx<4; idx++) {
				herder[idx].address=0;
				herder[idx].type=0;
				herder[idx].vmu=0;
			}
			inDemo=1;
			Game();
			nFrames=0;
			if (fMusic) {
				sound_start(title_song, false);
			}

			ReScanVMUs();
		}

		thread->pass();
		thread->sleep(10);	// the maple fails if you hit it too fast
	}

	if (conio != NULL) {
		conio->thaw();
	}

	sound_shutdown();
	sound->sfx_unload_all();
	thread->sleep(100);

	/* close our lib handles */
	lib_close(conio);
	lib_close(sound);
	lib_close(thread);
	lib_close(spu);
	lib_close(maple);
	lib_close(ta);
	lib_close(video);
	lib_close(fs);

	printf("Game complete.\n");
	printf("In game for %d seconds, drew %d frames, for %d fps\n", (nLastJiffy-nFirstJiffy)/100, nGameFrames, nGameFrames/((nLastJiffy-nFirstJiffy)/100));

	return 0;
}

/* adds the polygons required to put a 640x480 image, broken into 256x256 blocks, */
/* onto the 640x480 display. Draws 6 opaque triangle strips, 3 in each row. */
void AddPicture(uint32 txr_addr, float z) {
	vertex_ot_t vert ;
	poly_hdr_t poly ;

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 0;
	vert.y = 256;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 1.0f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 0;
	vert.y = 0;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 256;
	vert.y = 256;
	vert.u = 1.0f;
	vert.v = 1.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 256;
	vert.y = 0;
	vert.u = 1.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr+BLOCKSIZE, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 256;
	vert.y = 256;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 1.0f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 256;
	vert.y = 0;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 512;
	vert.y = 256;
	vert.u = 1.0f;
	vert.v = 1.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 512;
	vert.y = 0;
	vert.u = 1.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr+BLOCKSIZE*2, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 512;
	vert.y = 256;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 1.0f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 512;
	vert.y = 0;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 640;
	vert.y = 256;
	vert.u = 0.5f;
	vert.v = 1.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 640;
	vert.y = 0;
	vert.u = 0.5f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	// second row 

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr+BLOCKSIZE*3, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 0;
	vert.y = 480;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 0.875f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 0;
	vert.y = 256;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 256;
	vert.y = 480;
	vert.u = 1.0f;
	vert.v = 0.875f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 256;
	vert.y = 0;
	vert.u = 1.0f;
	vert.v = -1.0f;	// Why do I suddenly have to start doing *this*?
	ta->commit_vertex(&vert, sizeof(vert));

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr+BLOCKSIZE*4, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 256;
	vert.y = 480;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 0.875f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 256;
	vert.y = 256;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 512;
	vert.y = 480;
	vert.u = 1.0f;
	vert.v = 0.875f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 512;
	vert.y = 0;
	vert.u = 1.0f;
	vert.v = -1.0f;
	ta->commit_vertex(&vert, sizeof(vert));

	ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_ARGB1555, 256, 256, txr_addr+BLOCKSIZE*5, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = 512;
	vert.y = 480;
	vert.z = z;
	vert.u = 0.0f;
	vert.v = 0.875f;
	vert.a = 1.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 512;
	vert.y = 256;
	vert.u = 0.0f;
	vert.v = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = 640;
	vert.y = 480;
	vert.u = 0.5f;
	vert.v = 0.875f;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = 640;
	vert.y = 0;
	vert.u = 0.5f;
	vert.v = -1.0f;
	ta->commit_vertex(&vert, sizeof(vert));
}

void registerDummy(int typeTA)
{	// dummy triangle, just a point, to prevent an empty list
	vertex_oc_t vert ;
	poly_hdr_t poly ;

	ta->poly_hdr_col(&poly, typeTA);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = -1;
	vert.y = -1;
	vert.z = 0.0f;
	vert.a = 0.0f;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.flags = TA_VERTEX_EOL;
	ta->commit_vertex(&vert, sizeof(vert));
}

// Draw logos on VMUs 1-4, if present
void DoLogos() {
	static int nCount=0;
	int idx;

	for (idx=0; idx<4; idx++) {
		if (herder[idx].vmu!=0) {
			nCount++;
			if (nCount>=NUM_LOGOS) nCount=0;
			maple->vmu_draw_lcd(herder[idx].vmu, vmu_icon[nCount]);
		}
	}
}

void ReScanVMUs() {
	int idx;
	uint32 unit;
	
	// initialize player VMU addresses
	for (idx=0; idx<4; idx++) {
		herder[idx].address=0;
		herder[idx].type=0;
		herder[idx].vmu=0;
		ControllerState[idx]=0;

		for (unit=0; unit<6; unit++) {
			if (maple->device_func(idx, unit) & MAPLE_FUNC_CONTROLLER) {
				herder[idx].address=maple->create_addr(idx, unit);
				herder[idx].type=MAPLE_FUNC_CONTROLLER;
				ControllerState[idx]=1;
			}
			if (maple->device_func(idx, unit) & MAPLE_FUNC_LCD) {
				herder[idx].vmu=maple->create_addr(idx, unit);
			}
		}
	}
	DoLogos();
}

// Main game routine
// Return 0 to continue, or 1 to exit
int Game() {
	char *levelptr, *tmp;
	unsigned char tmp2;
	int x, y, idx, idx2, idx3, idx4, idx5;
	int speedloop;
	int gridx, gridy;
	int xstep, ystep ;
	SPRITE spr ;
	cont_cond_t cond ;
	int level ;
	int unit, fPaused;
	int exitflag, miscflag ;
	int lightning=0;
	uint32 desiredx, desiredy, currentx, currenty ;
	uint32 origx, origy ; 
	int nCountdown  = -1;
	int nGameCountdown=-1;
	int nWinner  = -1;

	// Some kind of initialization and menu needs to go here
	// For defining how many players, and what controller on each
	
	if (!inDemo) {
		if (doMenu()) {
			return 1;	// player wants out 
		}
	} else {
		thread->sleep(100);
	}

	// Now perform initialization
	for (idx=0; idx<4; idx++) {
		herder[idx].spr.txr_addr=txr_herder[idx];
		herder[idx].spr.tilenumber=0;
		herder[idx].spr.alpha=1.0;
		herder[idx].spr.x=(idx%2==0 ? GRIDSIZE+GRIDSIZE+PLAYFIELDXOFF : (GRIDSIZE*(LEVELXSIZE-3))+PLAYFIELDXOFF);
		herder[idx].spr.y=(idx<2 ? (3*GRIDSIZE)+PLAYFIELDYOFF : (GRIDSIZE*(LEVELYSIZE-3))+PLAYFIELDYOFF);
		herder[idx].spr.z=517.0;
		herder[idx].spr.xd=(idx%2==1 ? 1.0 : -1.0);
		herder[idx].spr.yd=0;
		herder[idx].spr.is3D=false;
		herder[idx].spr.ta_style=TA_TRANSLUCENT;
		herder[idx].destx=0;
		herder[idx].desty=0;
		herder[idx].score=0;
		herder[idx].stuntime=0;
		herder[idx].animframe=0;
		herder[idx].maxframes=4;
		herder[idx].sheep=0;
		herder[idx].speed=HERDERSPEED;
		herder[idx].range=1;
		herder[idx].charge=0;
		herder[idx].trap=0;
		herder[idx].pen=0;
		herder[idx].penhp=100;
		for (idx2=0; idx2<HISTORY; idx2++) {
			herder[idx].oldx[idx2]=herder[idx].spr.x;
			herder[idx].oldy[idx2]=herder[idx].spr.y;
		}
		herder[idx].oldidx=0;
		printf("Player %d initialized -> %s\n", idx, herder[idx].type==0?"Computer":"Human");
	}

	initSheep();

	// Game begin
	level=0;
	thread->sleep(500);
	nFirstJiffy=thread->jiffies();
	nGameFrames=0;

	// load a level
	memcpy(levelbuf, &Level[0], LEVELBYTES);
	levelptr=levelbuf;

	myBmp=load_bmp("level0a.bmp");
	if (NULL != myBmp) {
		printf("Drawing at 0x%08lx\r\n", txr_levela);
		ta->txr_load(txr_levela, myBmp->data, BLOCKSIZE);
		// Free the picture now that it's in video RAM
		free(myBmp);
	} else {
		printf("** Level0a didn't load!\n");
	}
	myBmp=load_bmp("level0b.bmp");
	if (NULL != myBmp) {
		printf("Drawing at 0x%08lx\r\n", txr_levelb);
		ta->txr_load(txr_levelb, myBmp->data, BLOCKSIZE);
		// Free the picture now that it's in video RAM
		free(myBmp);
	} else {
		printf("** Level0b didn't load!\n");
	}
	myBmp=load_bmp("level0c.bmp");
	if (NULL != myBmp) {
		printf("Drawing at 0x%08lx\r\n", txr_levelc);
		ta->txr_load(txr_levelc, myBmp->data, BLOCKSIZE);
		// Free the picture now that it's in video RAM
		free(myBmp);
	} else {
		printf("** Level0c didn't load!\n");
	}
	
	/* draw level */
	exitflag=0;
	fPaused=0;
	nGameCountdown=100;

	if (fMusic) {
		sound_start(mid_song, false);
	} else {
		sound_start(no_song, false);
	}

	while (1) {
		ta->begin_render();
		// (Opaque polygons)
		registerDummy(TA_OPAQUE);

		ta->commit_eol();

		// (Translucent polygons)

		// set up collision buffer
		clearbuffer();

		// Draw walls
		tmp=levelptr;
		spr.z=512.0f;
		spr.xd=0.0f;
		spr.yd=0.0f;
		spr.alpha=1.0f;
		spr.ta_style=TA_TRANSLUCENT;
		for (y=0; y<LEVELYSIZE; y++) {
			for (x=0; x<LEVELXSIZE; x++) {
				spr.txr_addr=txr_levela;
				spr.is3D=false;
				spr.tilenumber=7; spr.is3D=false;		// defaults
				spr.x=x*GRIDSIZE+PLAYFIELDXOFF;
				spr.y=y*GRIDSIZE+PLAYFIELDYOFF;

				tmp2=*(tmp++);
				
				if ((tmp2>='A')&&(tmp2<='J')) {	// Impassable, 3D
					spr.txr_addr=txr_levelb; spr.tilenumber=tmp2-'A'; spr.is3D=true;
					drawbuffer(spr.x, spr.y, TYPE_WALL);
					goto sort;
				}
				if ((tmp2>='K')&&(tmp2<='Z')) {	// Impassable, flat
					spr.txr_addr=txr_levelb; spr.tilenumber=tmp2-'A'; spr.is3D=false;
					drawbuffer(spr.x, spr.y, TYPE_WALL);
					goto sort;
				}
				if ((tmp2>='a')&&(tmp2<='z')) { // passable, flat
					spr.txr_addr=txr_levela; spr.tilenumber=tmp2-'a'; spr.is3D=false;
					goto sort;
				}
				if ((tmp2>='0')&&(tmp2<='9')) { // impassable, 3D, destructable
					spr.txr_addr=txr_levelc; spr.tilenumber=tmp2-'0'; spr.is3D=true;
					drawbuffer(spr.x, spr.y, TYPE_WALL);
					goto sort;
				}
				if (tmp2>127) {					// impassable, box animation
					tmp2--;
					*(tmp-1)=tmp2;
					if (((tmp2-128)%10)==0) {		// done
						if (rand()%10<=6) {
							*(tmp-1)=((tmp2-128)/10)+'a';		// normal tile
						} else {
							if (rand()%2) {
								*(tmp-1)=((tmp2-128)/10)+1;		// lightning tile
							} else {
								*(tmp-1)=((tmp2-128)/10)+11;	// speed tile
							}
						}
					}
					tmp2-=127;
					spr.txr_addr=txr_levelc; spr.tilenumber=tmp2/10; spr.is3D=true;
					spr.z+=0.1f; spr.alpha=8.0f;
					SortScaledSprite(&spr, (tmp2%10)/10.0f, (tmp2%10)/10.0f);
					spr.alpha=1.0f;	spr.z-=0.1f;

					drawbuffer(spr.x, spr.y, TYPE_WALL);
					spr.txr_addr=txr_levela; spr.tilenumber=tmp2/10; spr.is3D=false;
					goto sort;
				}
				if ((tmp2>=1)&&(tmp2<=10)) {	// Lightning powerup
					spr.txr_addr=txr_sprites; spr.tilenumber=8; spr.is3D=false;
					spr.z+=0.1f; spr.alpha=8.0f;
					SortSprite(&spr);
					spr.alpha-1.0f; spr.z-=0.1f;
					
					spr.txr_addr=txr_levela; spr.tilenumber=tmp2-1; spr.is3D=false;
					goto sort;
				}
				if ((tmp2>=11)&&(tmp2<=20)) {	// Speed powerup
					spr.txr_addr=txr_sprites; spr.tilenumber=13; spr.is3D=false;
					spr.z+=0.1f; spr.alpha=8.0f;
					SortSprite(&spr);
					spr.alpha-1.0f; spr.z-=0.1f;
					
					spr.txr_addr=txr_levela; spr.tilenumber=tmp2-11; spr.is3D=false;
					goto sort;
				}

sort:
				SortSprite(&spr);
			}
		}

		if (nGameCountdown>0) {
			nGameCountdown--;
			if (nGameCountdown>50) {
				// Ready! or Paused
				for (idx=0; idx<4; idx++) {
					SortSprite(&herder[idx].spr);
				}
				
				if (fPaused > 0) {
					addPage(txr_readygopause, 9, 207, 125, 240);
					nGameCountdown=52;
					for (idx=0; idx<4; idx++) {
						if (herder[idx].type == MAPLE_FUNC_CONTROLLER) {
							if (maple->cont_get_cond(herder[idx].address, &cond) == 0) {
								if (!(cond.buttons&CONT_START)) {
									if (fPaused==1) {
										fPaused=-(idx+1);
										nGameCountdown=0;
									}
								} else {
									if ((fPaused == idx+2)&&(cond.buttons&CONT_START)) {
										fPaused=1;
									}
								} 
							} else {
								printf("Controller read error.\n");
								thread->sleep(500);
								maple->rescan_bus(0);
								// Make sure we're up to date
								ReScanVMUs();
							}
						}
					}
				} else {
					addPage(txr_readygopause, 10, 1, 220, 51);
				}

				ta->commit_eol();
				ta->finish_frame();
				continue;
			}
			// Go!
			addPage(txr_readygopause, 0, 63, 255, 186);
		}

		// Put the players into the collision buffer
		// Sheep are handled separately
		for (idx=0; idx<4; idx++) {
			drawbuffer(herder[idx].spr.x, herder[idx].spr.y, TYPE_PLAY);
		}

		for (idx=0; idx<4; idx++) {
			for (speedloop=0; speedloop<(herder[idx].stuntime?2:1); speedloop++) {
				lightning=0;
				miscflag=0;			// flag if movement
				origx=herder[idx].spr.x;
				origy=herder[idx].spr.y;

				if (herder[idx].stuntime==0) {
					switch (herder[idx].type) {
					case MAPLE_FUNC_CONTROLLER:
						
						updateoldpos(&herder[idx]);

						if (maple->cont_get_cond(herder[idx].address, &cond) == 0) {
							herder[idx].spr.xd=0;
							herder[idx].spr.yd=0;
							if (!(cond.buttons & CONT_DPAD_UP)) {
								herder[idx].spr.yd=-1;
								herder[idx].spr.y-=herder[idx].speed;
								miscflag=1;
							}
							if (!(cond.buttons & CONT_DPAD_DOWN)) {
								herder[idx].spr.yd=1;
								herder[idx].spr.y+=herder[idx].speed;
								miscflag=1;
							}
							if (!(cond.buttons & CONT_DPAD_LEFT)) {
								herder[idx].spr.xd=-1;
								herder[idx].spr.x-=herder[idx].speed;
								miscflag=1;
							}
							if (!(cond.buttons & CONT_DPAD_RIGHT)) {
								herder[idx].spr.xd=1;
								herder[idx].spr.x+=herder[idx].speed;
								miscflag=1;
							}
							if ((!(cond.buttons & CONT_A)) ||
								(!(cond.buttons & CONT_B)) ||
								(!(cond.buttons & CONT_X)) ||
								(!(cond.buttons & CONT_Y))) {
								if (herder[idx].charge>=10) {
									lightning=1;			// flag to do lightning after we move
								}
							} else {
								herder[idx].charge++;
								// Adjust for maximum range
								if (herder[idx].charge > herder[idx].range*10+9) {
									herder[idx].charge=herder[idx].range*10+9;
								}

							}

							if ((cond.buttons & (CONT_A | CONT_B | CONT_X | CONT_Y)) == 0) {
								exitflag=-1;	// exit all
							}
							if (!(cond.buttons & CONT_START)) {
								// pause
								if (fPaused==0) {
									fPaused=idx+2;
									nGameCountdown=52;
								}
							} else {
								if (-(idx+1)==fPaused) {
									fPaused=0;
								}
							}

							break;
						} else {
							printf("Controller read error.\n");
							thread->sleep(500);
							maple->rescan_bus(0);
							// Make sure we're up to date
							ReScanVMUs();
						}

					case 0:		/* computer - no controller */
						// Check - if we didn't move, choose a new destination
						if ((herder[idx].oldx[herder[idx].oldidx]==herder[idx].spr.x)&&(herder[idx].oldy[herder[idx].oldidx]==herder[idx].spr.y)) {
							herder[idx].destx=0;
						}

						updateoldpos(&herder[idx]);

						if ((herder[idx].destx==0)||(herder[idx].desty==0)) {
							// new target - it doesn't have to be a legal destination,
							// as we'll pick a new one as soon as we get stuck :)
							herder[idx].destx=(rand()%20) * GRIDSIZE;
							herder[idx].desty=(rand()%15) * GRIDSIZE;
						}

						herder[idx].spr.xd=herder[idx].destx-herder[idx].spr.x;
						herder[idx].spr.yd=herder[idx].desty-herder[idx].spr.y;
						miscflag=1;

						if (herder[idx].spr.xd>0) {
							herder[idx].spr.x+=min(herder[idx].spr.xd, herder[idx].speed);
							herder[idx].spr.xd=1;
						} else {
							herder[idx].spr.x-=min(-herder[idx].spr.xd, herder[idx].speed);
							herder[idx].spr.xd=-1;
						}
						if (herder[idx].spr.yd>0) {
							herder[idx].spr.y+=min(herder[idx].spr.yd, herder[idx].speed);
							herder[idx].spr.yd=1;
						} else {
							herder[idx].spr.y-=min(-herder[idx].spr.yd, herder[idx].speed);
							herder[idx].spr.yd=-1;
						}

						// Lightning
						if ((rand()%10>8)&&(herder[idx].charge>10)) {
							lightning=1;
						} else {
							herder[idx].charge++;
							if (herder[idx].charge > herder[idx].range*10+9) {
								herder[idx].charge=herder[idx].range*10+9;
							}
						}

						break;

					default:	/* none - TODO: define a 'none' for not playing */
					}
				} else {
					// stunned
					updateoldpos(&herder[idx]);		// we still do this to keep sheep attached!

					herder[idx].stuntime--;
					herder[idx].spr.xd=herder[idx].destx-herder[idx].spr.x;
					herder[idx].spr.yd=herder[idx].desty-herder[idx].spr.y;
					miscflag=1;

					if (herder[idx].spr.xd>0) {
						herder[idx].spr.x+=min(herder[idx].spr.xd, HERDERMAXSPEED);
						herder[idx].spr.xd=1;
					} else {
						herder[idx].spr.x-=min(-herder[idx].spr.xd, HERDERMAXSPEED);
						herder[idx].spr.xd=-1;
					}
					if (herder[idx].spr.yd>0) {
						herder[idx].spr.y+=min(herder[idx].spr.yd, HERDERMAXSPEED);
						herder[idx].spr.yd=1;
					} else {
						herder[idx].spr.y-=min(-herder[idx].spr.yd, HERDERMAXSPEED);
						herder[idx].spr.yd=-1;
					}
				}
				
				if (miscflag) {
					if (nGameFrames%4==0) {
						herder[idx].animframe++;
						if (herder[idx].animframe >= herder[idx].maxframes) {
							herder[idx].animframe=0;
						}
					}
				
					// Check legality of new position, and shift towards legal spot
					// Save current position to reduce indirections!
					// No action if no movement
					// First erase self from collision buffer!
					drawbuffer(origx, origy, TYPE_NONE);

					currentx=herder[idx].spr.x;
					currenty=herder[idx].spr.y;
					gridx=((currentx-PLAYFIELDXOFF)+GRIDSIZE/2)/GRIDSIZE;
					gridy=((currenty-PLAYFIELDYOFF)+GRIDSIZE/2)/GRIDSIZE;
					xstep=(int)herder[idx].spr.xd;
					ystep=(int)herder[idx].spr.yd;
					desiredx=currentx;
					desiredy=currenty;

					if (xstep!=0) {
						// We're walking left or right.
						if (checkblock(currentx, origy,0)) {
							// we're walking into a wall
							desiredx=origx;
						}
					} else {
						desiredx=gridx*GRIDSIZE+PLAYFIELDXOFF;
					}
					if (ystep!=0) {
						// We're walking up or down
						if (checkblock(origx, currenty,0)) {
							desiredy=origy;
						}
					} else {
						desiredy=gridy*GRIDSIZE+PLAYFIELDYOFF;
					}
					if ((desiredx!=origx)&&(desiredy!=origy)) {
						// we're still trying to go diagonally, and both dirs are legal
						// We need to choose just one. We'd prefer the direction we
						// did not move last frame
						if (herder[idx].oldx[herder[idx].oldidx>0?herder[idx].oldidx-1:HISTORY-1]!=origx) {
							// we WERE going horizontal, so switch to vertical
							desiredx=origx;
						} else {
							// we WERE either going nowhere or vertical, take horizontal
							desiredy=origy;
						}
					}

					// Now adjust towards desired. We move at herder's speed to override movement
					// But not more pixels than are required :)
					if (herder[idx].stuntime>0) {
						if (desiredx>currentx) {
							herder[idx].spr.x+=min(desiredx-currentx, HERDERMAXSPEED+1);
						}
						if (desiredx<currentx) {
							herder[idx].spr.x-=min(currentx-desiredx, HERDERMAXSPEED+1);
						}
						if (desiredy>currenty) {
							herder[idx].spr.y+=min(desiredy-currenty, HERDERMAXSPEED+1);
						}
						if (desiredy<currenty) {
							herder[idx].spr.y-=min(currenty-desiredy, HERDERMAXSPEED+1);
						}
					} else {
						if (desiredx>currentx) {
							herder[idx].spr.x+=min(desiredx-currentx, herder[idx].speed+1);
						}
						if (desiredx<currentx) {
							herder[idx].spr.x-=min(currentx-desiredx, herder[idx].speed+1);
						}
						if (desiredy>currenty) {
							herder[idx].spr.y+=min(desiredy-currenty, herder[idx].speed+1);
						}
						if (desiredy<currenty) {
							herder[idx].spr.y-=min(currenty-desiredy, herder[idx].speed+1);
						}
					}
					// Done grid adjustment - make final legality check
					if (checkblock(herder[idx].spr.x, herder[idx].spr.y, 0)) {
						herder[idx].spr.x=origx;
						herder[idx].spr.y=origy;
					}

					// Check for powerup in new location
					gridx=((currentx-PLAYFIELDXOFF)+GRIDSIZE/2)/GRIDSIZE;
					gridy=((currenty-PLAYFIELDYOFF)+GRIDSIZE/2)/GRIDSIZE;

					tmp=levelptr + (gridy*LEVELXSIZE) + gridx;
					
					if ((*tmp>=1)&&(*tmp<=10)) {
						// lightning powerup
						*tmp=(*tmp+'a'-1);
						herder[idx].range++;
						if (herder[idx].range > MAXLIGHTNING) {
							herder[idx].range=MAXLIGHTNING;
						}
					}

					if ((*tmp>=11)&&(*tmp<=20)) {
						// speed powerup
						*tmp=(*tmp+'a'-11);
						herder[idx].speed++;
						if (herder[idx].range > HERDERMAXSPEED) {
							herder[idx].range=HERDERMAXSPEED;
						}
					}
				}

				herder[idx].spr.tilenumber=SHEEPDOWN;
				x=herder[idx].oldidx;
				if (x>0) {
					y=x-1;
				} else {
					y=HISTORY-1;
				}
				if (herder[idx].oldy[y]>herder[idx].oldy[x]) herder[idx].spr.tilenumber=SHEEPUP;
				if (herder[idx].oldx[y]>herder[idx].oldx[x]) herder[idx].spr.tilenumber=SHEEPLEFT;
				if (herder[idx].oldx[y]<herder[idx].oldx[x]) herder[idx].spr.tilenumber=SHEEPRIGHT;

				if ((herder[idx].oldy[y]==herder[idx].oldy[x])&&
					(herder[idx].oldx[y]==herder[idx].oldx[x])) {
					// We didn't move? Which way did we TRY to move? Any?
					if (herder[idx].spr.yd<0) herder[idx].spr.tilenumber=SHEEPUP;
					if (herder[idx].spr.xd<0) herder[idx].spr.tilenumber=SHEEPLEFT;
					if (herder[idx].spr.xd>0) herder[idx].spr.tilenumber=SHEEPRIGHT;
				}

				herder[idx].spr.tilenumber+=herder[idx].animframe*5;
				SortSprite(&herder[idx].spr);
				drawbuffer(herder[idx].spr.x, herder[idx].spr.y, TYPE_PLAY);

				spr.txr_addr=txr_sheep;
				spr.is3D=false;
				spr.z=514.0f;
				spr.alpha=0.7f;
				spr.ta_style=TA_TRANSLUCENT;
				x=herder[idx].oldidx-4;
				if (x<0) x+=HISTORY;
				if (x>0) {
					y=x-1;
				} else {
					y=HISTORY-1;
				}
				for (idx2=0; idx2<herder[idx].sheep; idx2++) {
					spr.tilenumber=SHEEPUP;
					if (herder[idx].oldy[y]<herder[idx].oldy[x]) spr.tilenumber=SHEEPDOWN;
					if (herder[idx].oldx[y]>herder[idx].oldx[x]) spr.tilenumber=SHEEPLEFT;
					if (herder[idx].oldx[y]<herder[idx].oldx[x]) spr.tilenumber=SHEEPRIGHT;
					spr.x=herder[idx].oldx[x];
					spr.y=herder[idx].oldy[x];
					spr.tilenumber+=((nGameFrames>>2)%SHEEPFRAMES)*5;
					SortSprite(&spr);
					x-=4;
					if (x<0) x+=HISTORY;
					if (x>0) {
						y=x-1;
					} else {
						y=HISTORY-1;
					}
				}

				while (lightning) {		// this is a while instead of an IF so we can break early
					// Fire lightning in the current direction
					// Lightning goes until it hits a wall, not affected by players or sheep
					x=herder[idx].spr.x;
					y=herder[idx].spr.y;
					if (herder[idx].spr.xd) {
						xstep=herder[idx].spr.xd*GRIDSIZE;
						ystep=0;
					} else {
						if (herder[idx].spr.yd) {
							ystep=herder[idx].spr.yd*GRIDSIZE;
							xstep=0;
						} else {
							// well, damn. No fire, then.
							break;
						}
					}

					spr.z=515.2f;
					spr.alpha=0.9f;
					spr.is3D=false;
					spr.ta_style=TA_TRANSLUCENT;
					if (nGameFrames%3==0) {
						miscflag=0;
					} else {
						miscflag=5;
					}

					for (idx2=0; idx2<herder[idx].charge/10; idx2++) {
						x+=xstep;
						y+=ystep;
						
						spr.txr_addr=txr_sprites;
						spr.tilenumber=(xstep==0?16:17)+miscflag;

						spr.x=x;
						spr.y=y;

						// Check against solid 3D walls to stop it
						tmp2=*(levelptr+(((y+GRIDSIZE/2)/GRIDSIZE)*LEVELXSIZE+((x+GRIDSIZE/2)/GRIDSIZE)));
						if ((tmp2>='A')&&(tmp2<='J')) break;	// done, don't draw this sprite

						// Fading destructible wall - stop on it
						if (tmp2>127) break;

						// Check against destructible walls, and nuke them :) (and stop the beam)
						if ((tmp2>='0')&&(tmp2<='9')) {
							*(levelptr+(((y+GRIDSIZE/2)/GRIDSIZE)*LEVELXSIZE+((x+GRIDSIZE/2)/GRIDSIZE)))=((tmp2-'0')*10+137);
							break;
						}

						// Draw the sprite 
						SortSprite(&spr);

						// Check for stunned sheep
						for (idx3=0; idx3<MAX_SHEEP; idx3++) {
							int x,y;
							if (sheep[idx3].type==1) {
								x=(int)sheep[idx3].spr.x;
								x-=spr.x;
								if (x<0) x=x*(-1);
								y=(int)sheep[idx3].spr.y;
								y-=spr.y;
								if (y<0) y=y*(-1);
								if ((x<28)&&(y<28)) {
									// got this sheep :)
									sheep[idx3].stuntime=60;	// 1 second stun time
									if ((fSound)&&(!sndCountdown)) {
										sound_effect(ZAPPED);
										sndCountdown=SNDDELAY;
									}
								}
							}
						}

						// How about other herders?
						for (idx3=0; idx3<4; idx3++) {
							int x,y;
							if ((idx3!=idx)&&(herder[idx3].type!=-1)) {	// don't check ourselves or non-players
								x=(int)herder[idx3].spr.x;
								x-=spr.x;
								if (x<0) x=x*(-1);
								y=(int)herder[idx3].spr.y;
								y-=spr.y;
								if (y<0) y=y*(-1);
								if ((x<28)&&(y<28)) {
									int tmpXS, tmpYS;
									if (xstep==0) {
										tmpXS=herder[idx3].spr.xd*GRIDSIZE;
									} else {
										tmpXS=xstep;
									}
									if (ystep==0) {
										tmpYS=herder[idx3].spr.yd*GRIDSIZE;
									} else {
										tmpYS=ystep;
									}
									// got this guy :)
									herder[idx3].stuntime=15;	// 1/4 second stun time
									herder[idx3].destx=herder[idx3].spr.x+tmpXS*2;
									herder[idx3].desty=herder[idx3].spr.y+tmpYS*2;
									// lose a sheep, if he has one, and the level isn't already over
									if ((herder[idx3].sheep>0)&&(nCountdown==-1)) {
										herder[idx3].sheep--;
										// find a free sheep to start
										for (idx4=0; idx4<MAX_SHEEP; idx4++) {
											if (sheep[idx4].type==0) {
												sheep[idx4].type=3;		// new sheep
												sheep[idx4].spr.x=herder[idx3].spr.x;
												sheep[idx4].spr.y=herder[idx3].spr.y;
												sheep[idx4].destx=sheep[idx4].spr.x+(xstep*5);
												sheep[idx4].desty=sheep[idx4].spr.y+(ystep*5);
												sheep[idx4].range=90;	// 1+1/2 seconds of freedom!
												sheep[idx4].spr.txr_addr=txr_sheep;
												sheep[idx4].spr.tilenumber=0;
												sheep[idx4].spr.alpha=1.0;
												sheep[idx4].spr.z=516.0;
												sheep[idx4].spr.xd=1;
												sheep[idx4].spr.yd=0;
												sheep[idx4].spr.is3D=false;
												sheep[idx4].spr.ta_style=TA_TRANSLUCENT;
												sheep[idx4].score=0;
												sheep[idx4].stuntime=0;
												sheep[idx4].animframe=0;
												sheep[idx4].maxframes=SHEEPFRAMES;
												sheep[idx4].sheep=0;
												sheep[idx4].trap=0;
												for (idx5=0; idx5<HISTORY; idx5++) {
													sheep[idx4].oldx[idx5]=sheep[idx4].spr.x;
													sheep[idx4].oldy[idx5]=sheep[idx4].spr.y;
												}
												sheep[idx4].oldidx=0;
												if ((fSound)&&(!sndCountdown)) {
													sound_effect(FREED);
													sndCountdown=SNDDELAY;
												}
												break;
											}
										}	// if we couldn't find one (somehow!), no worries
									}
								}
							}
						}
					}

					// countdown the charge
					herder[idx].charge--;
					
					break;		// this must be here to exit the lightning while! (no looping)
				}	// while
			}	// speed loop
		}	// idx loop

		moveSheep();

		// Check to see if any sheep we captured, and
		// check for end of level
		miscflag=0;
		for (idx=0; idx<MAX_SHEEP; idx++) {
			if (sheep[idx].type>0) {
				miscflag=1;
				if (sheep[idx].type==1) {
					for (idx2=0; idx2<4; idx2++) {
						if (herder[idx2].stuntime==0) {		// no sleepy herders
							x=(int)sheep[idx].spr.x;
							x-=herder[idx2].spr.x;
							if (x<0) x=x*(-1);
							y=(int)sheep[idx].spr.y;
							y-=herder[idx2].spr.y;
							if (y<0) y=y*(-1);
							if ((x<28)&&(y<28)) {
								// got this sheep :)
								sheep[idx].type=0;
								herder[idx2].sheep+=1;
								if ((fSound)&&(!sndCountdown)) {
									sound_effect(CAUGHT);
									sndCountdown=SNDDELAY;
								}
								break;
							}
						}
					}
				}
			}
		}
		
		if (-1!=nCountdown) {
			addPage(txr_winner[nWinner-1], 0, 0, 255, 255);
			nCountdown--;
		}

		if ((inDemo)&&(maple->first_controller()==0)) {
			// Attach Controller
			addPage(txr_controlwork, 0, 0, 192, 48);
		}

		// Work In Progress
		addPage2(txr_controlwork, 365, 370, 0, 96, 240, 191, nGameFrames%200-100);

		ta->commit_eol();
		ta->finish_frame();
		nGameFrames++;
		if (sndCountdown) sndCountdown--;

		if ((nCountdown==-1)&&(miscflag==0)) {
			// do level over!
			// Who won?
			x=0;
			for (idx=0; idx<4; idx++) {
				if (herder[idx].sheep>x) {
					y=idx;
					x=herder[idx].sheep;
				}
			}
			nWinner=y+1;
			nCountdown=450;		// 10 seconds
			if (fMusic) {
				sound_start(win_song, false);
			}
		}

		if (inDemo) {
			if (maple->cont_get_cond(maple->first_controller(), &cond) == 0) {
				if (!(cond.buttons & CONT_START)) {
					exitflag=1;
				}
			} else {
				printf("Controller read error.\n");
				thread->sleep(500);
				maple->rescan_bus(0);
				// Make sure we're up to date
				ReScanVMUs();
			}
		}

		if (nCountdown==0) {
			// end of a cycle
			break;
		}

		if (exitflag==-1) {
			nLastJiffy=thread->jiffies();
			inDemo=0;
			LoadProgram();
			return 1;
		}
		if (exitflag==1) {
			thread->sleep(500);
			break;
		}
	}
	nLastJiffy=thread->jiffies();
	inDemo=0;
	LoadProgram();
	return 0;
}

// Add the current position to the list of old positions
// And adjust the pointer. The list is a circular buffer
// HISTORY entries long. pHerd->oldidx is the current
// position, and pHerd->oldidx-1 is the previous position
void updateoldpos(pHerdStruct pHerd) {
	pHerd->oldidx++;
	if (pHerd->oldidx>=HISTORY) pHerd->oldidx=0;

	pHerd->oldx[pHerd->oldidx]=pHerd->spr.x;
	pHerd->oldy[pHerd->oldidx]=pHerd->spr.y;
}

// Attempt to work around some timing issues with VBLs
void blankscreen(int x) {
	int idx;

	for (idx=0; idx<x; idx++) {
		ta->begin_render();
		registerDummy(TA_OPAQUE);
		ta->commit_eol();
		registerDummy(TA_TRANSLUCENT);
		ta->commit_eol();
		ta->finish_frame();
	}
}

// Very simple menu
// Select players 1-4 as Human/Computer
// Rescan bus
// Start game
int doMenu() {
	int option;
	cont_cond_t cond;
	int first, idx;
	int lastbuttons;
	int nFrames;
	int bright;

	idx=0;
	while ((ControllerState[0]==0)&&(idx<4)) {
		idx++;
	}
	option=idx;
	cond.buttons=~(0);
	lastbuttons=~(CONT_START);
	first=1;
	nFrames=0;
	bright=75;

	while (1) {
		ta->begin_render();

		// Opaque polygons
		registerDummy(TA_OPAQUE);
		ta->commit_eol();

		// Translucent polygons
		if (maple->first_controller()==0) {
			// Attach Controller
			addPage(txr_controlwork, 0, 0, 192, 48);
			ta->commit_eol();
			ta->finish_frame();
			thread->sleep(500);
			maple->rescan_bus(0);
			// Make sure we're up to date
			ReScanVMUs();
			continue;
		}
		
		bright++;
		if (bright>100) bright=75;

		CenterDrawFont(1, 24, 100, "Cool Herders");
		CenterDrawFont(0, -1, 100, "Internal WIP 2001");

		DrawFont(0, 128, 110, option==0 ? bright : ControllerState[0]==0? 25 : 50, herder[0].type==0 ? "1: Computer" : "1: Human   ");
		DrawFont(0, 336, 110, option==1 ? bright : ControllerState[1]==0? 25 : 50, herder[1].type==0 ? "2: Computer" : "2: Human   ");
		DrawFont(0, 128, 132, option==2 ? bright : ControllerState[2]==0? 25 : 50, herder[2].type==0 ? "3: Computer" : "3: Human   ");
		DrawFont(0, 336, 132, option==3 ? bright : ControllerState[3]==0? 25 : 50, herder[3].type==0 ? "4: Computer" : "4: Human   ");
		CenterDrawFont(0, -1, option==4 ? bright : 50, "Redetect Pads");
		CenterDrawFont(0, -1, option==5 ? bright : 50, fMusic ? "Music: On " : "Music: Off");
		CenterDrawFont(0, -1, option==6 ? bright : 50, fSound ? "Sound: On " : "Sound: Off");

		CenterDrawFont(0, 260, 100, "Press A to select or START to begin");
		CenterDrawFont(0, -1, 100, "Press A&B&X&Y to exit demo");

		CenterDrawFont(0, 320, 75, "Cool Herders");
		CenterDrawFont(0, -1, 75, "* 2001 Moving Target Software Design");
		CenterDrawFont(0, -1, 75, "www.mtsdhq.com");
		CenterDrawFont(0, -1, 75, "This product is a Work In Progress");
		CenterDrawFont(0, -1, 75, "Beta build 26 July 2001");
		CenterDrawFont(0, -1, 75, "Not intended for distribution");

		ta->commit_eol();

		ta->finish_frame();

		nFrames++;

		if ((nFrames%180)==0) {
			DoLogos();
		} else {
			if (first) {
				thread->sleep(300);	// hacky
				first=0;
			}

			if (maple->cont_get_cond(maple->first_controller(), &cond) < 0) {
				printf("Controller read error.\n");
				thread->sleep(500);
				maple->rescan_bus(0);
				// Make sure we're up to date
				ReScanVMUs();
				continue;
			}

			if (lastbuttons != cond.buttons) {
				if (!(cond.buttons & CONT_START)) {
					blankscreen(1);
					spu->disable();
					if ((herder[0].type==0)&&(herder[1].type==0)&&(herder[2].type==0)&&(herder[3].type==0)) {
						inDemo=1;
					}
					return 0;
				}

				if ((cond.buttons & (CONT_A | CONT_B | CONT_X | CONT_Y)) == 0) {
					printf("Exit...\n");
					return 1;
				}

				if (cond.buttons & CONT_DPAD_DOWN) {
					do {
						option--;
						if (option<0) option=6;
					} while ((option<4)&&(ControllerState[option]==0));
				}

				if (cond.buttons & CONT_DPAD_UP) {
					do {
						option++;
						if (option>6) option=0;
					} while ((option<4)&&(ControllerState[option]==0));
				}

				if (!(cond.buttons & CONT_A)) {
					switch (option) {
					case 4:
						maple->rescan_bus(1);
						// Make sure we're up to date
						ReScanVMUs();
						blankscreen(5);
						break;
					case 5:
						if (fMusic) {
							fMusic=0;
							sound_start(no_song, false);
						} else {
							fMusic=1;
							sound_start(title_song, false);
						}
						break;
					case 6:
						if (fSound) {
							fSound=0;
						} else {
							fSound=1;
						}
						break;
					case 0:
					case 1:
					case 2:
					case 3:
						if (herder[option].type==0) {
							herder[option].type=MAPLE_FUNC_CONTROLLER;
						} else {
							herder[option].type=0;
						}
					}
				}

				lastbuttons=cond.buttons;
			}
		}
	}
}

