/****************************************/
/* Cool Herders                         */
/* A demented muse by Brendan Wiese     */
/* http://starbow.org                   */
/* Programmed by Mike Brent             */
/* http://tursi.yiffco.com              */
/* disclaimer.c                         */
/****************************************/

/* 
   Boot screens and loader for Cool Herders
   Derived from:
   KallistiOS 0.90
   (c)2001 Dan Potter
*/

#include <stdio.h>
#include <malloc.h>
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/fs.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/spu.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/sndsrv_abi.h>
#include <kallisti/irq.h>
#include <kallisti/vmu.h>
#include <kallisti/biosfont.h>
#include "../kosh/abi/conio.h"
#include "sprite.h"
#include "cool.h"
#include "bmp.h"
#include "sheep.h"
#include "sound.h"

void load_data();

extern uint32 txr_title, txr_sprites, txr_winner[4];
extern uint32 txr_levela, txr_levelb, txr_levelc;
extern uint32 txr_sheep, txr_smfont, txr_lgfont;
extern uint32 txr_readygopause, txr_herder[4];
extern uint32 txr_controlwork;

/* Builds a little disclaimer texture (easier than doing a bunch of font
   polygons) */
static uint32 disc_txr;

static void disc_texture() {
	uint16	*txrtmp;
	int	i, x, y = (480 - 11*24)/2;
	char	*msg[] = {
"Now Loading:  Cool Herders",
"",
"(C) 2001 Moving Target Software Design",
"Concept and Art by Brendan 'Binky' Wiese",
"Programming by Mike 'Tursi' Brent",
"Original Music by Sean Connell",
"Sound FX by Flossie,", 
"the Prettiest Sheep in the World",
"Thanks to Yak, Gord, NeoGeoMan, and Bard",
"",
"http://www.mtsdhq.com"
	};

	/* Allocate texture space */
	disc_txr = ta->txr_allocate(512*512*2);
	txrtmp = (uint16*)ta->txr_map(disc_txr);

	memset(ta->txr_map(disc_txr), 0, 512*512*2);

	/* Draw a message into it */
	for (i=0; i<11; i++) {
		x = 256 - (strlen(msg[i]) * 12)/2;
		video->bfont_draw_str(txrtmp + y*512+x, 512, 0, msg[i]);
		y += 25;
	}
}

static void disc_display(float fade) {
	vertex_ot_t	v;
	poly_hdr_t	hdr;

	if (fade < 0.0f) fade = 0.0f;

	ta->poly_hdr_txr(&hdr, TA_OPAQUE, TA_RGB565, 512, 512, disc_txr, TA_NO_FILTER);
	ta->commit_poly_hdr(&hdr);

	/* - -
	   + - */
	v.flags = TA_VERTEX_NORMAL;
	v.x = 64.0f;
	v.y = 480.0f;
	v.z = 1.0f;
	v.u = 0.0f; v.v = 480.0f/512.0f;
	v.a = 1.0f; v.r = v.g = v.b = fade;
	v.oa = v.or = v.og = v.ob = 0.0f;
	ta->commit_vertex(&v, sizeof(v));

	/* + -
	   - - */
	v.y = 0.0f;
	v.v = 0.0f;
	ta->commit_vertex(&v, sizeof(v));

	/* - -
	   - + */
	v.x = 640.0f - 64.0f;
	v.y = 480.0f;
	v.u = 1.0f; v.v = 480.0f / 512.0f;
	ta->commit_vertex(&v, sizeof(v));

	/* - +
	   - - */
	v.flags = TA_VERTEX_EOL;
	v.y = 0.0f;
	v.v = 0.0f;
	ta->commit_vertex(&v, sizeof(v));
}

static void disc_blank() {
	poly_hdr_t	hdr;

	ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
	ta->commit_poly_hdr(&hdr);
}

void do_screen() {
	float f;
	
	/* Setup the texture */
	disc_texture();

	/* Fade in the disclaimer */
	for (f=0.0f; f<1.0f; f+=0.01f) {
		ta->begin_render();
		disc_display(f);
		ta->commit_eol();
		disc_blank();
		ta->commit_eol();
		ta->finish_frame();
	}

	/* load data here. this provides delay */
	load_data();

	/* Fade out the disclaimer */
	for (f=1.0f; f>=-0.03f; f-=0.01f) {
		ta->begin_render();
		disc_display(f);
		ta->commit_eol();
		disc_blank();
		ta->commit_eol();
		ta->finish_frame();
	}

	// Reallocate the text buffer for level tiles :)
	// allocate memory for the 3 (for now) pages of background tiles
	txr_levela=disc_txr;
	txr_levelb=disc_txr+BLOCKSIZE;
	txr_levelc=disc_txr+BLOCKSIZE+BLOCKSIZE;

	// One free slot left? :)

}

uint32 pic_load(char *fn) {
	BITMAP *myBmp;
	uint32 addr;

	myBmp=load_bmp(fn);
	if (NULL != myBmp) {
		addr=ta->txr_allocate(myBmp->nXBlocks*myBmp->nYBlocks*BLOCKSIZE);

		printf("Drawing at 0x%08lx\r\n", addr);
		ta->txr_load(addr, myBmp->data, myBmp->nXBlocks*myBmp->nYBlocks*BLOCKSIZE);
	
		// Free the picture now that it's in video RAM
		free(myBmp);
	} else {
		printf("** %s didn't load!\n", fn);
		addr=0;
	}
	return addr;
}


void load_data() {
	// Thanks for local directory, Dan!
	int idx;
	char buf[256];

	txr_title=pic_load("title.bmp");
	txr_sprites=pic_load("sprites.bmp");
	
	for (idx=0; idx<4; idx++) {
		strcpy(buf, "p?_win.bmp");
		buf[1]=idx+'1';
		txr_winner[idx]=pic_load(buf);	
	}

	txr_sheep=pic_load("sheep.bmp");
	txr_herder[0]=pic_load("rd_herd.bmp");
	txr_herder[1]=pic_load("yl_herd.bmp");
	txr_herder[2]=pic_load("bl_herd.bmp");
	txr_herder[3]=pic_load("pu_herd.bmp");
	txr_smfont=pic_load("font2.bmp");
	txr_lgfont=pic_load("font1.bmp");
	txr_readygopause=pic_load("readygopause.bmp");
	txr_controlwork=pic_load("controlwork.bmp");

	sound_init();

}
